@@ -1,13 +1,10 @@
 package Myriad::Exception;
 
-use strict;
-use warnings;
+use Myriad::Class type => 'role';
 
-our $VERSION = '0.010'; # VERSION
+our $VERSION = '1.000'; # VERSION
 our $AUTHORITY = 'cpan:DERIV'; # AUTHORITY
 
-use utf8;
-
 =encoding utf8
 
 =head1 NAME
@@ -20,12 +17,8 @@ This is a rôle used for all exceptions throughout the framework.
 
 =cut
 
-no indirect qw(fatal);
-
-use Scalar::Util;
-use Role::Tiny;
-
-requires qw(category message);
+method category;
+method message;
 
 =head2 throw
 
@@ -33,9 +26,8 @@ Instantiates a new exception and throws it (by calling L<perlfunc/die>).
 
 =cut
 
-sub throw {
-    my $self = shift;
-    $self = $self->new(@_) unless Scalar::Util::blessed($self);
+sub throw ($class, @args) {
+    my $self = blessed($class) ? $class : $class->new(@args);
     die $self;
 }
 
@@ -49,5 +41,5 @@ See L<Myriad/CONTRIBUTORS> for full details.
 
 =head1 LICENSE
 
-Copyright Deriv Group Services Ltd 2020-2021. Licensed under the same terms as Perl itself.
+Copyright Deriv Group Services Ltd 2020-2022. Licensed under the same terms as Perl itself.
 
