@@ -1,13 +1,10 @@
 package Myriad::Exception::Builder;
 
-use strict;
-use warnings;
+use Myriad::Class class => '';
 
-our $VERSION = '0.010'; # VERSION
+our $VERSION = '1.000'; # VERSION
 our $AUTHORITY = 'cpan:DERIV'; # AUTHORITY
 
-use utf8;
-
 =encoding utf8
 
 =head1 NAME
@@ -20,10 +17,14 @@ See L<Myriad::Exception> for the rôle that defines the exception API.
 
 =cut
 
-no indirect qw(fatal);
+# We deliberately *don't* want class/method keywords, but *do* want MOP
+use Object::Pad qw(:experimental);
+
 use Myriad::Exception;
 use Myriad::Exception::Base;
 
+use constant CLASS_CREATION_METHOD => Object::Pad::MOP::Class->can('create_class') || Object::Pad::MOP::Class->can('begin_class');
+
 # Not currently used, but nice as a hint
 our @EXPORT = our @EXPORT_OK = qw(declare_exception);
 
@@ -31,6 +32,8 @@ our @EXPORT = our @EXPORT_OK = qw(declare_exception);
 # repetition when there's a long list of exceptions to be defining
 our %DEFAULT_CATEGORY_FOR_CLASS;
 
+our %EXCEPTIONS;
+
 sub import {
     my ($class, %args) = @_;
     my $pkg = caller;
@@ -85,29 +88,63 @@ Returns the generated classname.
 sub declare_exception {
     my ($name, %args) = @_;
     my $caller = caller;
-
-    my $pkg = join '::', (
+    my $pkg = length($name) ? (join '::', (
         delete($args{package}) || ('Myriad::Exception::' . (caller =~ s{^Myriad::}{}r))
-    ), $name;
-
-    no strict 'refs';
-    push @{$pkg . '::ISA'}, qw(Myriad::Exception::Base);
+    ), $name) : $caller;
     my $category = delete $args{category} // $DEFAULT_CATEGORY_FOR_CLASS{$caller};
     die 'invalid category ' . $category unless $category =~ /^[0-9a-z_]+$/;
-    *{$pkg . '::category'} = sub { $category };
     my $message = delete $args{message} // 'unknown';
-    *{$pkg . '::message'} = sub {
-        my $self = shift;
-        my $str = $message . ' (category=' . $self->category;
-        if($self->reason) {
-            $str .= ' , reason=' . $self->reason;
-        }
 
-        $str . ')';
-    };
-    Role::Tiny->apply_roles_to_package(
-        $pkg => 'Myriad::Exception'
-    )
+    die 'already have exception ' . $pkg if exists $EXCEPTIONS{$pkg};
+
+    $EXCEPTIONS{$pkg} //= create_exception({
+        package  => $pkg,
+        category => $category,
+        message  => $message
+    });
+    return $pkg;
+}
+
+sub create_exception ($details) {
+    my $pkg = delete $details->{package} or die 'no package';
+    my $category = delete $details->{category} or die 'no category';
+    my $message = delete $details->{message} or die 'no message';
+
+    try {
+        Myriad::Class->import(
+            target  => $pkg,
+            class   => '',
+        );
+        $EXCEPTIONS{$pkg} = my $class = (CLASS_CREATION_METHOD)->(
+            'Object::Pad::MOP::Class',
+            $pkg,
+            extends => 'Myriad::Exception::Base',
+        );
+        $class->add_role('Myriad::Exception');
+        $class->add_method(
+            category => sub ($self) { $category }
+        );
+        $class->add_method(
+            message => sub ($self) {
+                my $str = $message . ' (category=' . $self->category;
+                if($self->reason) {
+                    $str .= ' , reason=' . $self->reason;
+                }
+                return $str . ')';
+            }
+        );
+        { # Until we get class methods in role { } blocks, need to inject this directly
+            no strict 'refs';
+            *{$pkg . '::throw'} = sub ($class, @args) {
+                my $self = blessed($class) ? $class : $class->new(@args);
+                die $self;
+            };
+        }
+        $class->seal;
+        return $class;
+    } catch ($e) {
+        $log->errorf('Failed to raise declare exception - %s', $e);
+    }
 }
 
 1;
@@ -120,5 +157,5 @@ See L<Myriad/CONTRIBUTORS> for full details.
 
 =head1 LICENSE
 
-Copyright Deriv Group Services Ltd 2020-2021. Licensed under the same terms as Perl itself.
+Copyright Deriv Group Services Ltd 2020-2022. Licensed under the same terms as Perl itself.
 
