@@ -2,7 +2,7 @@ package Myriad::RPC::Message;
 
 use Myriad::Class;
 
-our $VERSION = '0.010'; # VERSION
+our $VERSION = '1.000'; # VERSION
 our $AUTHORITY = 'cpan:DERIV'; # AUTHORITY
 
 =encoding utf8
@@ -247,6 +247,16 @@ sub apply_decoding ($data, $encoding) {
     }
 }
 
+=head2 passed_deadline
+
+Check if the message is stil relevent
+
+=cut
+
+method passed_deadline {
+    time > $deadline ? 1 : 0;
+}
+
 1;
 
 =head1 AUTHOR
@@ -257,5 +267,5 @@ See L<Myriad/CONTRIBUTORS> for full details.
 
 =head1 LICENSE
 
-Copyright Deriv Group Services Ltd 2020-2021. Licensed under the same terms as Perl itself.
+Copyright Deriv Group Services Ltd 2020-2022. Licensed under the same terms as Perl itself.
 
