@@ -1,13 +1,10 @@
 package Myriad::Role::Subscription;
 
-use strict;
-use warnings;
+use Myriad::Class type => 'role';
 
-our $VERSION = '0.010'; # VERSION
+our $VERSION = '1.000'; # VERSION
 our $AUTHORITY = 'cpan:DERIV'; # AUTHORITY
 
-use utf8;
-
 =encoding utf8
 
 =head1 NAME
@@ -35,13 +32,6 @@ a concrete implementation - instead, see classes such as:
 
 =cut
 
-no indirect qw(fatal);
-use Future::AsyncAwait;
-
-use experimental qw(signatures);
-
-use Role::Tiny;
-
 =head1 METHODS
 
 =head2 create_from_sink
@@ -62,9 +52,9 @@ it takes a hash as an argument that should have the following
 
 =cut
 
-requires 'create_from_sink';
+method create_from_sink;
 
-=head2 create_from_sink
+=head2 create_from_source
 
 Register a new C<Emitter> to receive events from.
 
@@ -80,7 +70,7 @@ it takes a hash as an argument that should have the following
 
 =cut
 
-requires 'create_from_source';
+method create_from_source;
 
 =head2 start
 
@@ -88,7 +78,7 @@ Start processing the subscriptions.
 
 =cut
 
-requires 'start';
+method start;
 
 =head2 stop
 
@@ -96,7 +86,7 @@ Stop processing the subscriptions.
 
 =cut
 
-requires 'stop';
+method stop;
 
 1;
 
@@ -110,5 +100,5 @@ See L<Myriad/CONTRIBUTORS> for full details.
 
 =head1 LICENSE
 
-Copyright Deriv Group Services Ltd 2020-2021. Licensed under the same terms as Perl itself.
+Copyright Deriv Group Services Ltd 2020-2022. Licensed under the same terms as Perl itself.
 
