@@ -2,7 +2,7 @@ package Myriad::Service::Storage::Remote;
 
 use Myriad::Class;
 
-our $VERSION = '0.010'; # VERSION
+our $VERSION = '1.000'; # VERSION
 our $AUTHORITY = 'cpan:DERIV'; # AUTHORITY
 
 =encoding utf8
@@ -40,7 +40,7 @@ BEGIN {
             return $self->storage->$method($self->apply_prefix($key), @rest)->on_ready(sub {
                 my $f = shift;
                 $metrics->report_timer(time_elapsed =>
-                    $f->elapsed, {method => $method, status => $f->state, service => $self->local_service_name});
+                    $f->elapsed // 0, {method => $method, status => $f->state, service => $self->local_service_name});
             });
         });
     }
@@ -91,6 +91,6 @@ See L<Myriad/CONTRIBUTORS> for full details.
 
 =head1 LICENSE
 
-Copyright Deriv Group Services Ltd 2020-2021. Licensed under the same terms as Perl itself.
+Copyright Deriv Group Services Ltd 2020-2022. Licensed under the same terms as Perl itself.
 
 
