@@ -1,14 +1,10 @@
 package Myriad::Subscription::Implementation::Memory;
 
-our $VERSION = '0.010'; # VERSION
+use Myriad::Class extends => 'IO::Async::Notifier', does => [ 'Myriad::Role::Subscription', 'Myriad::Util::Defer' ];
+
+our $VERSION = '1.000'; # VERSION
 our $AUTHORITY = 'cpan:DERIV'; # AUTHORITY
 
-use Myriad::Class extends => qw(IO::Async::Notifier);
-
-use Role::Tiny::With;
-
-with 'Myriad::Role::Subscription';
-
 has $transport;
 
 has $receivers;
@@ -16,6 +12,9 @@ has $receivers;
 has $should_shutdown = 0;
 has $stopped;
 
+# FIXME Need to update :Defer for Object::Pad
+sub MODIFY_CODE_ATTRIBUTES { }
+
 BUILD {
     $receivers = [];
 }
@@ -35,6 +34,7 @@ async method create_from_source (%args) {
     my $src          = delete $args{source} or die 'need a source';
     my $service      = delete $args{service} or die 'need a service';
     my $channel_name = $service . '.' . $args{channel};
+    await $transport->create_stream($channel_name);
 
     $src->map(async sub {
         my $message = shift;
@@ -49,29 +49,28 @@ async method create_from_source (%args) {
 async method create_from_sink (%args) {
     my $sink = delete $args{sink} or die 'need a sink';
     my $remote_service = $args{from} || $args{service};
+    my $service_name = $args{service};
     my $channel_name = $remote_service . '.' . $args{channel};
 
     push $receivers->@*, {
-        channel => $channel_name,
-        sink    => $sink
+        channel    => $channel_name,
+        sink       => $sink,
+        group_name => $service_name,
+        group      => 0
     };
     return;
 }
 
+async method create_group ($subscription) {
+    return if $subscription->{group};
+    await $transport->create_consumer_group($subscription->{channel}, $subscription->{group_name}, 0, 0);
+    $subscription->{group} = 1;
+}
 
 async method start {
-    my %groups_for_channel;
     while (1) {
         await &fmap_void($self->$curry::curry(async method ($subscription) {
-            unless(exists $groups_for_channel{$subscription->{channel}}{subscriber}) {
-                try {
-                    await $transport->create_consumer_group($subscription->{channel}, 'subscriber', 0, 1);
-                    $groups_for_channel{$subscription->{channel}}{subscriber} = 1;
-                } catch ($e) {
-                    $log->tracef('Failed to create consumer group due: %s', $e);
-                }
-            }
-
+            await $self->create_group($subscription);
             try {
                 $log->tracef('Sink blocked state: %s', $subscription->{sink}->unblocked->state);
                 await Future->wait_any(
@@ -85,12 +84,12 @@ async method start {
 
             my $messages = await $transport->read_from_stream_by_consumer(
                 $subscription->{channel},
-                'subscriber',
+                $subscription->{group_name},
                 'consumer'
             );
             for my $event_id (sort keys $messages->%*) {
                 $subscription->{sink}->emit($messages->{$event_id});
-                await $transport->ack_message($subscription->{channel}, 'subscriber', $event_id);
+                await $transport->ack_message($subscription->{channel}, $subscription->{group_name}, $event_id);
             }
 
             if($should_shutdown) {
@@ -119,5 +118,5 @@ See L<Myriad/CONTRIBUTORS> for full details.
 
 =head1 LICENSE
 
-Copyright Deriv Group Services Ltd 2020-2021. Licensed under the same terms as Perl itself.
+Copyright Deriv Group Services Ltd 2020-2022. Licensed under the same terms as Perl itself.
 
