@@ -1,23 +1,19 @@
 package Myriad::Subscription::Implementation::Redis;
 
-our $VERSION = '0.010'; # VERSION
+use Myriad::Class extends => qw(IO::Async::Notifier), does => [
+    'Myriad::Role::Subscription'
+];
+
+our $VERSION = '1.000'; # VERSION
 our $AUTHORITY = 'cpan:DERIV'; # AUTHORITY
 
-use Myriad::Class extends => qw(IO::Async::Notifier);
-
-use JSON::MaybeUTF8 qw(:v1);
-use Unicode::UTF8 qw(decode_utf8 encode_utf8);
 use Myriad::Util::UUID;
 
-use Role::Tiny::With;
-
 use constant MAX_ALLOWED_STREAM_LENGTH => 10_000;
 
-with 'Myriad::Role::Subscription';
-
 has $redis;
 
-has $uuid;
+has $client_id;
 
 # A list of all sources that emits events to Redis
 # will need to keep track of them to block them when
@@ -30,7 +26,7 @@ has @receivers;
 has $should_shutdown;
 
 BUILD {
-    $uuid = Myriad::Util::UUID::uuid();
+    $client_id = Myriad::Util::UUID::uuid();
 }
 
 method configure (%args) {
@@ -44,19 +40,20 @@ async method create_from_source (%args) {
 
     my $stream = "service.subscriptions.$service/$args{channel}";
 
+    $log->tracef('Adding subscription source %s to handler', $stream);
+    push @emitters, {
+        stream => $stream,
+        source => $src,
+        max_len => $args{max_len} // MAX_ALLOWED_STREAM_LENGTH
+    };
     $src->unblocked->then($self->$curry::weak(async method {
         # The streams will be checked later by "check_for_overflow" to avoid unblocking the source by mistake
         # we will make "check_for_overflow" aware about this stream after the service has started
-        push @emitters, {
-            stream => $stream,
-            source => $src,
-            max_len => $args{max_len} // MAX_ALLOWED_STREAM_LENGTH
-        };
-        await $src->map($self->$curry::weak(method {
-            $log->tracef('sub has an event! %s', $_);
+        await $src->map($self->$curry::weak(method ($event) {
+            $log->tracef('Subscription source %s adding an event: %s',$stream, $event);
             return $redis->xadd(
                 encode_utf8($stream) => '*',
-                data => encode_json_utf8($_),
+                data => encode_json_utf8($event),
             );
         }))->ordered_futures(
             low => 100,
@@ -78,18 +75,19 @@ async method create_from_sink (%args) {
         or die 'need a sink';
     my $remote_service = $args{from} || $args{service};
     my $stream = "service.subscriptions.$remote_service/$args{channel}";
-    $log->tracef('created sub thing from sink');
+    $log->tracef('Adding subscription sink %s to handler', $stream);
     push @receivers, {
-        key    => $stream,
-        client => $args{client},
-        sink   => $sink,
-        group  => 0,
+        key        => $stream,
+        sink       => $sink,
+        group_name => $args{service},
+        group      => 0,
     };
 }
 
 async method start {
     $should_shutdown //= $self->loop->new_future(label => 'subscription::redis::shutdown');
-    $log->tracef('Starting subscription handler');
+    $log->tracef('Starting subscription handler client_id: %s', $client_id);
+    await $self->create_streams;
     await Future->wait_any(
         $should_shutdown->without_cancel,
         $self->receive_items,
@@ -105,70 +103,67 @@ async method stop {
 
 async method create_group($receiver) {
     unless ($receiver->{group}) {
-        await $redis->create_group($receiver->{key}, $uuid);
+        await $redis->create_group($receiver->{key}, $receiver->{group_name});
         $receiver->{group} = 1;
     }
     return;
 }
 
 async method receive_items {
-    $log->tracef('Start loop for receiving items');
-    while (1) {
-        if(@receivers) {
-            my $item = shift @receivers;
-            push @receivers, $item;
+    $log->tracef('Start receiving from (%d) subscription sinks', scalar(@receivers));
+    while (@receivers == 0) {
+        $log->tracef('No receivers, waiting for a few seconds');
+        await $self->loop->delay_future(after => 5);
+    }
 
-            $log->tracef('Will readgroup on %s', $item->{key});
-            my $stream = $item->{key};
-            my $sink = $item->{sink};
-            my $client = $item->{client};
+    await &fmap_void($self->$curry::curry(async method ($rcv) {
+        my $stream     = $rcv->{key};
+        my $sink       = $rcv->{sink};
+        my $group_name = $rcv->{group_name};
 
+        while (1) {
             try {
-                await Future->wait_any(
-                    $self->loop->timeout_future(after => 0.5),
-                    $sink->unblocked,
-                );
-            } catch {
-                $log->tracef("skipped stream %s because sink is blocked", $stream);
-                next
+                await $self->create_group($rcv);
+            } catch ($e) {
+                $log->warnf('skipped subscription on stream %s because: %s will try again', $stream, $e);
+                await $self->loop->delay_future(after => 5);
+                next;
             }
-
-            await $self->create_group($item);
-
+            await $sink->unblocked;
             my @events = await $redis->read_from_stream(
                 stream => $stream,
-                group => $uuid,
-                client => $client
+                group  => $group_name,
+                client => $client_id
             );
 
             for my $event (@events) {
                 try {
-                    my $event_data = $event->{data}->[1];
+                    my $event_data = decode_json_utf8($event->{data}->[1]);
+                    $log->tracef('Passing event: %s | from stream: %s to subscription sink: %s', $event_data, $stream, $sink->label);
                     $sink->source->emit({
-                        data => decode_json_utf8($event_data)
+                        data => $event_data
                     });
                 } catch($e) {
                     $log->tracef(
-                        "An error happened while decoding event data for stream %s message: %s, error: %s",
+                        'An error happened while decoding event data for stream %s message: %s, error: %s',
                         $stream,
                         $event->{data},
                         $e
                     );
                 }
+
                 await $redis->ack(
                     $stream,
-                    $uuid,
+                    $group_name,
                     $event->{id}
                 );
             }
-        } else {
-            $log->tracef('No receivers, waiting for a few seconds');
-            await $self->loop->delay_future(after => 5);
         }
-    }
+    }), foreach => [@receivers], concurrent => scalar @receivers);
 }
 
 async method check_for_overflow () {
+    $log->tracef('Start checking overflow for (%d) subscription sources', scalar(@emitters));
     while (1) {
         if(@emitters) {
             my $emitter = shift @emitters;
@@ -178,7 +173,7 @@ async method check_for_overflow () {
                 if ($len >= $emitter->{max_len}) {
                     unless ($emitter->{source}->is_paused) {
                         $emitter->{source}->pause;
-                        $log->tracef("Paused emitter on %s, length is %s, max allowed %s", $emitter->{stream}, $len, $emitter->{max_len});
+                        $log->tracef('Paused subscription source on %s, length is %s, max allowed %s', $emitter->{stream}, $len, $emitter->{max_len});
                     }
                     await $redis->cleanup(
                         stream => $emitter->{stream},
@@ -187,11 +182,11 @@ async method check_for_overflow () {
                 } else {
                     if($emitter->{source}->is_paused) {
                         $emitter->{source}->resume;
-                        $log->infof("Resumed emitter on %s, length is %s", $emitter->{stream}, $len);
+                        $log->infof('Resumed subscription source on %s, length is %s', $emitter->{stream}, $len);
                     }
                 }
             } catch ($e) {
-                $log->warnf("An error ocurred while trying to check on stream %s status - %s", $emitter->{stream}, $e);
+                $log->warnf('An error ocurred while trying to check on stream %s status - %s', $emitter->{stream}, $e);
             }
         }
 
@@ -200,6 +195,10 @@ async method check_for_overflow () {
     }
 }
 
+async method create_streams() {
+    await Future->needs_all(map { $redis->create_stream($_->{stream}) } @emitters);
+}
+
 1;
 
 __END__
