@@ -1,17 +1,10 @@
 package Myriad::Transport::HTTP;
 
-use strict;
-use warnings;
+use Myriad::Class extends => 'IO::Async::Notifier';
 
-our $VERSION = '0.010'; # VERSION
+our $VERSION = '1.000'; # VERSION
 our $AUTHORITY = 'cpan:DERIV'; # AUTHORITY
 
-use Object::Pad;
-
-class Myriad::Transport::HTTP extends IO::Async::Notifier;
-
-use curry;
-
 use Net::Async::HTTP;
 use Net::Async::HTTP::Server;
 
@@ -65,5 +58,5 @@ Deriv Group Services Ltd. C<< DERIV@cpan.org >>
 
 =head1 LICENSE
 
-Copyright Deriv Group Services Ltd 2020-2021. Licensed under the same terms as Perl itself.
+Copyright Deriv Group Services Ltd 2020-2022. Licensed under the same terms as Perl itself.
 
