@@ -1,15 +1,10 @@
 package Myriad::Util::Defer;
 
-use strict;
-use warnings;
+use Myriad::Class type => 'role';
 
-our $VERSION = '0.010'; # VERSION
+our $VERSION = '1.000'; # VERSION
 our $AUTHORITY = 'cpan:DERIV'; # AUTHORITY
 
-use utf8;
-
-no indirect qw(fatal);
-
 =encoding utf8
 
 =head1 NAME
@@ -25,39 +20,50 @@ and defaults to no delay.
 
 =cut
 
-use Future::AsyncAwait;
 use Attribute::Handlers;
 use Class::Method::Modifiers;
 
-use Exporter qw(import export_to_level);
-use Log::Any qw($log);
-
-our @IMPORT = our @IMPORT_OK = qw(Defer);
-
 use constant RANDOM_DELAY => $ENV{MYRIAD_RANDOM_DELAY} || 0;
 
+use Sub::Util;
+sub MODIFY_CODE_ATTRIBUTES ($class, $code, @attrs) {
+    my $name = Sub::Util::subname($code);
+    my ($method_name) = $name =~ m{::([^:]+)$};
+    for my $attr (@attrs) {
+        if($attr eq 'Defer') {
+            $class->defer_method($method_name, $name);
+        } else {
+            die 'unknown attribute ' . $attr;
+        }
+    }
+    return;
+}
+
 # Helper method that allows us to return a not-quite-immediate
 # Future from some inherently non-async code.
-sub Defer : ATTR(CODE) {
-    my ($package, $symbol, $referent, $attr, $data, $phase, $filename, $linenum) = @_;
-    my $name = *{$symbol}{NAME} or die 'need a symbol name';
-    $log->tracef('will defer handler for %s::%s', $package, $name);
-    around join('::', $package, $name) => async sub {
-        my ($code, $self, @args) = @_;
-
+sub defer_method ($package, $name, $fqdn) {
+    $log->tracef('will defer handler for %s::%s by %f', $package, $name, RANDOM_DELAY);
+    my $code = $package->can($name);
+    my $replacement = async sub ($self, @args) {
         # effectively $loop->later, but in an await-compatible way:
         # either zero (default behaviour) or if we have a random
         # delay assigned, use that to drive a uniform rand() call
+        $log->tracef('call to %s::%s, deferring start', $package, $name);
         await $self->loop->delay_future(
             after => rand(RANDOM_DELAY)
         );
 
-        $log->tracef('deferred call to %s::%s', $package, $name);
+        $log->tracef('deferred call to %s::%s runs now', $package, $name);
 
         return await $self->$code(
             @args
         );
-    } if RANDOM_DELAY;
+    };
+    {
+        no strict 'refs';
+        no warnings 'redefine';
+        *{join '::', $package, $name} = $replacement if RANDOM_DELAY;
+    }
 }
 
 1;
@@ -70,5 +76,5 @@ See L<Myriad/CONTRIBUTORS> for full details.
 
 =head1 LICENSE
 
-Copyright Deriv Group Services Ltd 2020-2021. Licensed under the same terms as Perl itself.
+Copyright Deriv Group Services Ltd 2020-2022. Licensed under the same terms as Perl itself.
 
